@@ -1,14 +1,16 @@
 use std::fmt::Debug;
+use std::str::FromStr;
+use std::sync::Arc;
 
 use enum_dispatch::enum_dispatch;
 use jiter::{PartialMode, StringCacheMode};
 
-use pyo3::exceptions::PyTypeError;
+use pyo3::exceptions::{PyTypeError, PyValueError};
 use pyo3::types::{PyAny, PyDict, PyString, PyTuple, PyType};
 use pyo3::{intern, PyTraverseError, PyVisit};
 use pyo3::{prelude::*, IntoPyObjectExt};
 
-use crate::build_tools::{py_schema_err, py_schema_error_type};
+use crate::build_tools::{py_schema_err, py_schema_error_type, ExtraBehavior};
 use crate::definitions::{Definitions, DefinitionsBuilder};
 use crate::errors::{LocItem, ValError, ValResult, ValidationError};
 use crate::input::{Input, InputType, StringMapping};
@@ -60,7 +62,7 @@ mod timedelta;
 mod tuple;
 mod typed_dict;
 mod union;
-mod url;
+pub(crate) mod url;
 mod uuid;
 mod validation_state;
 mod with_default;
@@ -71,11 +73,11 @@ pub use with_default::DefaultType;
 #[pyclass(module = "pydantic_core._pydantic_core", name = "Some")]
 pub struct PySome {
     #[pyo3(get)]
-    value: PyObject,
+    value: Py<PyAny>,
 }
 
 impl PySome {
-    fn new(value: PyObject) -> Self {
+    fn new(value: Py<PyAny>) -> Self {
         Self { value }
     }
 }
@@ -87,7 +89,7 @@ impl PySome {
     }
 
     #[new]
-    pub fn py_new(value: PyObject) -> Self {
+    pub fn py_new(value: Py<PyAny>) -> Self {
         Self { value }
     }
 
@@ -106,14 +108,14 @@ impl PySome {
 #[pyclass(module = "pydantic_core._pydantic_core", frozen)]
 #[derive(Debug)]
 pub struct SchemaValidator {
-    validator: CombinedValidator,
-    definitions: Definitions<CombinedValidator>,
+    validator: Arc<CombinedValidator>,
+    definitions: Definitions<Arc<CombinedValidator>>,
     // References to the Python schema and config objects are saved to enable
     // reconstructing the object for cloudpickle support (see `__reduce__`).
     py_schema: Py<PyAny>,
     py_config: Option<Py<PyDict>>,
     #[pyo3(get)]
-    title: PyObject,
+    title: Py<PyAny>,
     hide_input_in_errors: bool,
     validation_error_cause: bool,
     cache_str: StringCacheMode,
@@ -159,25 +161,31 @@ impl SchemaValidator {
     }
 
     #[allow(clippy::too_many_arguments)]
-    #[pyo3(signature = (input, *, strict=None, from_attributes=None, context=None, self_instance=None, allow_partial=PartialMode::Off, by_alias=None, by_name=None))]
+    #[pyo3(signature = (input, *, strict=None, extra=None, from_attributes=None, context=None, self_instance=None, allow_partial=PartialMode::Off, by_alias=None, by_name=None))]
     pub fn validate_python(
         &self,
         py: Python,
         input: &Bound<'_, PyAny>,
         strict: Option<bool>,
+        extra: Option<&Bound<'_, PyString>>,
         from_attributes: Option<bool>,
         context: Option<&Bound<'_, PyAny>>,
         self_instance: Option<&Bound<'_, PyAny>>,
         allow_partial: PartialMode,
         by_alias: Option<bool>,
         by_name: Option<bool>,
-    ) -> PyResult<PyObject> {
+    ) -> PyResult<Py<PyAny>> {
+        let extra_behavior = extra
+            .map(|e| ExtraBehavior::from_str(e.to_str()?).map_err(|err| PyValueError::new_err(err.to_string())))
+            .transpose()?;
+
         #[allow(clippy::used_underscore_items)]
         self._validate(
             py,
             input,
             InputType::Python,
             strict,
+            extra_behavior,
             from_attributes,
             context,
             self_instance,
@@ -189,24 +197,30 @@ impl SchemaValidator {
     }
 
     #[allow(clippy::too_many_arguments)]
-    #[pyo3(signature = (input, *, strict=None, from_attributes=None, context=None, self_instance=None, by_alias=None, by_name=None))]
+    #[pyo3(signature = (input, *, strict=None, extra=None, from_attributes=None, context=None, self_instance=None, by_alias=None, by_name=None))]
     pub fn isinstance_python(
         &self,
         py: Python,
         input: &Bound<'_, PyAny>,
         strict: Option<bool>,
+        extra: Option<&Bound<'_, PyString>>,
         from_attributes: Option<bool>,
         context: Option<&Bound<'_, PyAny>>,
         self_instance: Option<&Bound<'_, PyAny>>,
         by_alias: Option<bool>,
         by_name: Option<bool>,
     ) -> PyResult<bool> {
+        let extra_behavior = extra
+            .map(|e| ExtraBehavior::from_str(e.to_str()?).map_err(|err| PyValueError::new_err(err.to_string())))
+            .transpose()?;
+
         #[allow(clippy::used_underscore_items)]
         match self._validate(
             py,
             input,
             InputType::Python,
             strict,
+            extra_behavior,
             from_attributes,
             context,
             self_instance,
@@ -223,18 +237,23 @@ impl SchemaValidator {
     }
 
     #[allow(clippy::too_many_arguments)]
-    #[pyo3(signature = (input, *, strict=None, context=None, self_instance=None, allow_partial=PartialMode::Off, by_alias=None, by_name=None))]
+    #[pyo3(signature = (input, *, strict=None, extra=None, context=None, self_instance=None, allow_partial=PartialMode::Off, by_alias=None, by_name=None))]
     pub fn validate_json(
         &self,
         py: Python,
         input: &Bound<'_, PyAny>,
         strict: Option<bool>,
+        extra: Option<&Bound<'_, PyString>>,
         context: Option<&Bound<'_, PyAny>>,
         self_instance: Option<&Bound<'_, PyAny>>,
         allow_partial: PartialMode,
         by_alias: Option<bool>,
         by_name: Option<bool>,
-    ) -> PyResult<PyObject> {
+    ) -> PyResult<Py<PyAny>> {
+        let extra_behavior = extra
+            .map(|e| ExtraBehavior::from_str(e.to_str()?).map_err(|err| PyValueError::new_err(err.to_string())))
+            .transpose()?;
+
         let r = match json::validate_json_bytes(input) {
             #[allow(clippy::used_underscore_items)]
             Ok(v_match) => self._validate_json(
@@ -242,6 +261,7 @@ impl SchemaValidator {
                 input,
                 v_match.into_inner().as_slice(),
                 strict,
+                extra_behavior,
                 context,
                 self_instance,
                 allow_partial,
@@ -254,19 +274,23 @@ impl SchemaValidator {
     }
 
     #[allow(clippy::too_many_arguments)]
-    #[pyo3(signature = (input, *, strict=None, context=None, allow_partial=PartialMode::Off, by_alias=None, by_name=None))]
+    #[pyo3(signature = (input, *, strict=None, extra=None, context=None, allow_partial=PartialMode::Off, by_alias=None, by_name=None))]
     pub fn validate_strings(
         &self,
         py: Python,
         input: Bound<'_, PyAny>,
         strict: Option<bool>,
+        extra: Option<&Bound<'_, PyString>>,
         context: Option<&Bound<'_, PyAny>>,
         allow_partial: PartialMode,
         by_alias: Option<bool>,
         by_name: Option<bool>,
-    ) -> PyResult<PyObject> {
+    ) -> PyResult<Py<PyAny>> {
         let t = InputType::String;
         let string_mapping = StringMapping::new_value(input).map_err(|e| self.prepare_validation_err(py, e, t))?;
+        let extra_behavior = extra
+            .map(|e| ExtraBehavior::from_str(e.to_str()?).map_err(|err| PyValueError::new_err(err.to_string())))
+            .transpose()?;
 
         #[allow(clippy::used_underscore_items)]
         match self._validate(
@@ -274,6 +298,7 @@ impl SchemaValidator {
             &string_mapping,
             t,
             strict,
+            extra_behavior,
             None,
             context,
             None,
@@ -287,7 +312,7 @@ impl SchemaValidator {
     }
 
     #[allow(clippy::too_many_arguments)]
-    #[pyo3(signature = (obj, field_name, field_value, *, strict=None, from_attributes=None, context=None, by_alias=None, by_name=None))]
+    #[pyo3(signature = (obj, field_name, field_value, *, strict=None, extra=None,from_attributes=None, context=None, by_alias=None, by_name=None))]
     pub fn validate_assignment(
         &self,
         py: Python,
@@ -295,15 +320,21 @@ impl SchemaValidator {
         field_name: &str,
         field_value: Bound<'_, PyAny>,
         strict: Option<bool>,
+        extra: Option<&Bound<'_, PyString>>,
         from_attributes: Option<bool>,
         context: Option<&Bound<'_, PyAny>>,
         by_alias: Option<bool>,
         by_name: Option<bool>,
-    ) -> PyResult<PyObject> {
+    ) -> PyResult<Py<PyAny>> {
+        let extra_behavior = extra
+            .map(|e| ExtraBehavior::from_str(e.to_str()?).map_err(|err| PyValueError::new_err(err.to_string())))
+            .transpose()?;
+
         let extra = Extra {
             input_type: InputType::Python,
             data: None,
             strict,
+            extra_behavior,
             from_attributes,
             field_name: Some(PyString::new(py, field_name)),
             context,
@@ -326,11 +357,12 @@ impl SchemaValidator {
         py: Python,
         strict: Option<bool>,
         context: Option<&Bound<'_, PyAny>>,
-    ) -> PyResult<PyObject> {
+    ) -> PyResult<Py<PyAny>> {
         let extra = Extra {
             input_type: InputType::Python,
             data: None,
             strict,
+            extra_behavior: None,
             from_attributes: None,
             field_name: None,
             context,
@@ -388,17 +420,19 @@ impl SchemaValidator {
         input: &(impl Input<'py> + ?Sized),
         input_type: InputType,
         strict: Option<bool>,
+        extra_behavior: Option<ExtraBehavior>,
         from_attributes: Option<bool>,
         context: Option<&Bound<'py, PyAny>>,
         self_instance: Option<&Bound<'py, PyAny>>,
         allow_partial: PartialMode,
         by_alias: Option<bool>,
         by_name: Option<bool>,
-    ) -> ValResult<PyObject> {
+    ) -> ValResult<Py<PyAny>> {
         let mut recursion_guard = RecursionState::default();
         let mut state = ValidationState::new(
             Extra::new(
                 strict,
+                extra_behavior,
                 from_attributes,
                 context,
                 self_instance,
@@ -420,12 +454,13 @@ impl SchemaValidator {
         input: &Bound<'_, PyAny>,
         json_data: &[u8],
         strict: Option<bool>,
+        extra_behavior: Option<ExtraBehavior>,
         context: Option<&Bound<'_, PyAny>>,
         self_instance: Option<&Bound<'_, PyAny>>,
         allow_partial: PartialMode,
         by_alias: Option<bool>,
         by_name: Option<bool>,
-    ) -> ValResult<PyObject> {
+    ) -> ValResult<Py<PyAny>> {
         let json_value = jiter::JsonValue::parse_with_config(json_data, true, allow_partial)
             .map_err(|e| json::map_json_err(input, e, json_data))?;
         #[allow(clippy::used_underscore_items)]
@@ -434,6 +469,7 @@ impl SchemaValidator {
             &json_value,
             InputType::Json,
             strict,
+            extra_behavior,
             None,
             context,
             self_instance,
@@ -464,8 +500,8 @@ pub trait BuildValidator: Sized {
     fn build(
         schema: &Bound<'_, PyDict>,
         config: Option<&Bound<'_, PyDict>>,
-        definitions: &mut DefinitionsBuilder<CombinedValidator>,
-    ) -> PyResult<CombinedValidator>;
+        definitions: &mut DefinitionsBuilder<Arc<CombinedValidator>>,
+    ) -> PyResult<Arc<CombinedValidator>>;
 }
 
 /// Logic to create a particular validator, called in the `validator_match` macro, then in turn by `build_validator`
@@ -473,8 +509,8 @@ fn build_specific_validator<T: BuildValidator>(
     val_type: &str,
     schema_dict: &Bound<'_, PyDict>,
     config: Option<&Bound<'_, PyDict>>,
-    definitions: &mut DefinitionsBuilder<CombinedValidator>,
-) -> PyResult<CombinedValidator> {
+    definitions: &mut DefinitionsBuilder<Arc<CombinedValidator>>,
+) -> PyResult<Arc<CombinedValidator>> {
     T::build(schema_dict, config, definitions)
         .map_err(|err| py_schema_error_type!("Error building \"{}\" validator:\n  {}", val_type, err))
 }
@@ -497,25 +533,25 @@ macro_rules! validator_match {
 pub fn build_validator_base(
     schema: &Bound<'_, PyAny>,
     config: Option<&Bound<'_, PyDict>>,
-    definitions: &mut DefinitionsBuilder<CombinedValidator>,
-) -> PyResult<CombinedValidator> {
+    definitions: &mut DefinitionsBuilder<Arc<CombinedValidator>>,
+) -> PyResult<Arc<CombinedValidator>> {
     build_validator_inner(schema, config, definitions, false)
 }
 
 pub fn build_validator(
     schema: &Bound<'_, PyAny>,
     config: Option<&Bound<'_, PyDict>>,
-    definitions: &mut DefinitionsBuilder<CombinedValidator>,
-) -> PyResult<CombinedValidator> {
+    definitions: &mut DefinitionsBuilder<Arc<CombinedValidator>>,
+) -> PyResult<Arc<CombinedValidator>> {
     build_validator_inner(schema, config, definitions, true)
 }
 
 fn build_validator_inner(
     schema: &Bound<'_, PyAny>,
     config: Option<&Bound<'_, PyDict>>,
-    definitions: &mut DefinitionsBuilder<CombinedValidator>,
+    definitions: &mut DefinitionsBuilder<Arc<CombinedValidator>>,
     use_prebuilt: bool,
-) -> PyResult<CombinedValidator> {
+) -> PyResult<Arc<CombinedValidator>> {
     let dict = schema.downcast::<PyDict>()?;
     let py = schema.py();
     let type_: Bound<'_, PyString> = dict.get_as_req(intern!(py, "type"))?;
@@ -524,7 +560,7 @@ fn build_validator_inner(
     if use_prebuilt {
         // if we have a SchemaValidator on the type already, use it
         if let Ok(Some(prebuilt_validator)) = prebuilt::PrebuiltValidator::try_get_from_schema(type_, dict) {
-            return Ok(prebuilt_validator);
+            return Ok(Arc::new(prebuilt_validator));
         }
     }
 
@@ -636,6 +672,9 @@ pub struct Extra<'a, 'py> {
     pub data: Option<Bound<'py, PyDict>>,
     /// whether we're in strict or lax mode
     pub strict: Option<bool>,
+    /// Whether to ignore, allow, or forbid extra data during model validation
+    #[allow(clippy::struct_field_names)]
+    pub extra_behavior: Option<ExtraBehavior>,
     /// Validation time setting of `from_attributes`
     pub from_attributes: Option<bool>,
     /// context used in validator functions
@@ -656,6 +695,7 @@ impl<'a, 'py> Extra<'a, 'py> {
     #[allow(clippy::too_many_arguments)]
     pub fn new(
         strict: Option<bool>,
+        extra_behavior: Option<ExtraBehavior>,
         from_attributes: Option<bool>,
         context: Option<&'a Bound<'py, PyAny>>,
         self_instance: Option<&'a Bound<'py, PyAny>>,
@@ -668,6 +708,7 @@ impl<'a, 'py> Extra<'a, 'py> {
             input_type,
             data: None,
             strict,
+            extra_behavior,
             from_attributes,
             field_name: None,
             context,
@@ -685,6 +726,7 @@ impl Extra<'_, '_> {
             input_type: self.input_type,
             data: self.data.clone(),
             strict: Some(true),
+            extra_behavior: self.extra_behavior,
             from_attributes: self.from_attributes,
             field_name: self.field_name.clone(),
             context: self.context,
@@ -809,7 +851,7 @@ pub trait Validator: Send + Sync + Debug {
         py: Python<'py>,
         input: &(impl Input<'py> + ?Sized),
         state: &mut ValidationState<'_, 'py>,
-    ) -> ValResult<PyObject>;
+    ) -> ValResult<Py<PyAny>>;
 
     /// Get a default value, currently only used by `WithDefaultValidator`
     fn default_value<'py>(
@@ -817,7 +859,7 @@ pub trait Validator: Send + Sync + Debug {
         _py: Python<'py>,
         _outer_loc: Option<impl Into<LocItem>>,
         _state: &mut ValidationState<'_, 'py>,
-    ) -> ValResult<Option<PyObject>> {
+    ) -> ValResult<Option<Py<PyAny>>> {
         Ok(None)
     }
 
@@ -830,7 +872,7 @@ pub trait Validator: Send + Sync + Debug {
         _field_name: &str,
         _field_value: &Bound<'py, PyAny>,
         _state: &mut ValidationState<'_, 'py>,
-    ) -> ValResult<PyObject> {
+    ) -> ValResult<Py<PyAny>> {
         let py_err = PyTypeError::new_err(format!("validate_assignment is not supported for {}", self.get_name()));
         Err(py_err.into())
     }
