@@ -1,3 +1,5 @@
+use std::sync::Arc;
+
 use pyo3::exceptions::PyKeyError;
 use pyo3::intern;
 use pyo3::prelude::*;
@@ -22,7 +24,7 @@ struct Field {
     name: String,
     lookup_key_collection: LookupKeyCollection,
     name_py: Py<PyString>,
-    validator: CombinedValidator,
+    validator: Arc<CombinedValidator>,
     frozen: bool,
 }
 
@@ -33,8 +35,8 @@ pub struct ModelFieldsValidator {
     fields: Vec<Field>,
     model_name: String,
     extra_behavior: ExtraBehavior,
-    extras_validator: Option<Box<CombinedValidator>>,
-    extras_keys_validator: Option<Box<CombinedValidator>>,
+    extras_validator: Option<Arc<CombinedValidator>>,
+    extras_keys_validator: Option<Arc<CombinedValidator>>,
     strict: bool,
     from_attributes: bool,
     loc_by_alias: bool,
@@ -48,8 +50,8 @@ impl BuildValidator for ModelFieldsValidator {
     fn build(
         schema: &Bound<'_, PyDict>,
         config: Option<&Bound<'_, PyDict>>,
-        definitions: &mut DefinitionsBuilder<CombinedValidator>,
-    ) -> PyResult<CombinedValidator> {
+        definitions: &mut DefinitionsBuilder<Arc<CombinedValidator>>,
+    ) -> PyResult<Arc<CombinedValidator>> {
         let py = schema.py();
 
         let strict = is_strict(schema, config)?;
@@ -59,12 +61,12 @@ impl BuildValidator for ModelFieldsValidator {
         let extra_behavior = ExtraBehavior::from_schema_or_config(py, schema, config, ExtraBehavior::Ignore)?;
 
         let extras_validator = match (schema.get_item(intern!(py, "extras_schema"))?, &extra_behavior) {
-            (Some(v), ExtraBehavior::Allow) => Some(Box::new(build_validator(&v, config, definitions)?)),
+            (Some(v), ExtraBehavior::Allow) => Some(build_validator(&v, config, definitions)?),
             (Some(_), _) => return py_schema_err!("extras_schema can only be used if extra_behavior=allow"),
             (_, _) => None,
         };
         let extras_keys_validator = match (schema.get_item(intern!(py, "extras_keys_schema"))?, &extra_behavior) {
-            (Some(v), ExtraBehavior::Allow) => Some(Box::new(build_validator(&v, config, definitions)?)),
+            (Some(v), ExtraBehavior::Allow) => Some(build_validator(&v, config, definitions)?),
             (Some(_), _) => return py_schema_err!("extras_keys_schema can only be used if extra_behavior=allow"),
             (_, _) => None,
         };
@@ -99,7 +101,7 @@ impl BuildValidator for ModelFieldsValidator {
             });
         }
 
-        Ok(Self {
+        Ok(CombinedValidator::ModelFields(Self {
             fields,
             model_name,
             extra_behavior,
@@ -110,7 +112,7 @@ impl BuildValidator for ModelFieldsValidator {
             loc_by_alias: config.get_as(intern!(py, "loc_by_alias"))?.unwrap_or(true),
             validate_by_alias: config.get_as(intern!(py, "validate_by_alias"))?,
             validate_by_name: config.get_as(intern!(py, "validate_by_name"))?,
-        }
+        })
         .into())
     }
 }
@@ -126,11 +128,12 @@ impl Validator for ModelFieldsValidator {
         py: Python<'py>,
         input: &(impl Input<'py> + ?Sized),
         state: &mut ValidationState<'_, 'py>,
-    ) -> ValResult<PyObject> {
+    ) -> ValResult<Py<PyAny>> {
         // this validator does not yet support partial validation, disable it to avoid incorrect results
         state.allow_partial = false.into();
 
         let strict = state.strict_or(self.strict);
+        let extra_behavior = state.extra_behavior_or(self.extra_behavior);
         let from_attributes = state.extra().from_attributes.unwrap_or(self.from_attributes);
 
         // we convert the DictType error to a ModelType error
@@ -167,12 +170,12 @@ impl Validator for ModelFieldsValidator {
 
         // we only care about which keys have been used if we're iterating over the object for extra after
         // the first pass
-        let mut used_keys: Option<AHashSet<&str>> =
-            if self.extra_behavior == ExtraBehavior::Ignore || dict.is_py_get_attr() {
-                None
-            } else {
-                Some(AHashSet::with_capacity(self.fields.len()))
-            };
+        let mut used_keys: Option<AHashSet<&str>> = if extra_behavior == ExtraBehavior::Ignore || dict.is_py_get_attr()
+        {
+            None
+        } else {
+            Some(AHashSet::with_capacity(self.fields.len()))
+        };
 
         {
             let state = &mut state.rebind_extra(|extra| extra.data = Some(model_dict.clone()));
@@ -351,13 +354,13 @@ impl Validator for ModelFieldsValidator {
                 used_keys,
                 errors: &mut errors,
                 fields_set_vec: &mut fields_set_vec,
-                extra_behavior: self.extra_behavior,
+                extra_behavior,
                 extras_validator: self.extras_validator.as_deref(),
                 extras_keys_validator: self.extras_keys_validator.as_deref(),
                 state,
             })??;
 
-            if matches!(self.extra_behavior, ExtraBehavior::Allow) {
+            if matches!(extra_behavior, ExtraBehavior::Allow) {
                 model_extra_dict_op = Some(model_extra_dict);
             }
         }
@@ -370,7 +373,7 @@ impl Validator for ModelFieldsValidator {
 
             // if we have extra=allow, but we didn't create a dict because we were validating
             // from attributes, set it now so __pydantic_extra__ is always a dict if extra=allow
-            if matches!(self.extra_behavior, ExtraBehavior::Allow) && model_extra_dict_op.is_none() {
+            if matches!(extra_behavior, ExtraBehavior::Allow) && model_extra_dict_op.is_none() {
                 model_extra_dict_op = Some(PyDict::new(py));
             }
 
@@ -385,15 +388,16 @@ impl Validator for ModelFieldsValidator {
         field_name: &str,
         field_value: &Bound<'py, PyAny>,
         state: &mut ValidationState<'_, 'py>,
-    ) -> ValResult<PyObject> {
+    ) -> ValResult<Py<PyAny>> {
         let dict = obj.downcast::<PyDict>()?;
+        let extra_behavior = state.extra_behavior_or(self.extra_behavior);
 
         let get_updated_dict = |output: &Bound<'py, PyAny>| {
             dict.set_item(field_name, output)?;
             Ok(dict)
         };
 
-        let prepare_result = |result: ValResult<PyObject>| match result {
+        let prepare_result = |result: ValResult<Py<PyAny>>| match result {
             Ok(output) => get_updated_dict(&output.into_bound(py)),
             Err(ValError::LineErrors(line_errors)) => {
                 let errors = line_errors
@@ -409,7 +413,7 @@ impl Validator for ModelFieldsValidator {
         let data_dict = dict.copy()?;
         if let Err(err) = data_dict.del_item(field_name) {
             // KeyError is fine here as the field might not be in the dict
-            if !err.get_type(py).is(&PyType::new::<PyKeyError>(py)) {
+            if !err.get_type(py).is(PyType::new::<PyKeyError>(py)) {
                 return Err(err.into());
             }
         }
@@ -435,7 +439,7 @@ impl Validator for ModelFieldsValidator {
                 // to determine how to handle assignment
                 // For models / typed dicts we forbid assigning extra attributes
                 // unless the user explicitly set extra_behavior to 'allow'
-                match self.extra_behavior {
+                match extra_behavior {
                     ExtraBehavior::Allow => match self.extras_validator {
                         Some(ref validator) => prepare_result(validator.validate(py, field_value, state))?,
                         None => get_updated_dict(field_value)?,
@@ -454,7 +458,7 @@ impl Validator for ModelFieldsValidator {
             }
         };
 
-        let new_extra = match &self.extra_behavior {
+        let new_extra = match &extra_behavior {
             ExtraBehavior::Allow => {
                 let non_extra_data = PyDict::new(py);
                 self.fields.iter().try_for_each(|f| -> PyResult<()> {
